@@ -29,7 +29,7 @@ class Tensor_attributes {
     // In approach 1, users provide a value to embed into the graph.
     // In approach 2, users set is_pass_by_value boolean and then pass a pointer to scalar value with execute() API.
     // A closed set of types that are allowed to be passed by value.
-    using pass_by_values_t = std::variant<int32_t, half, float, nv_bfloat16>;
+    using pass_by_values_t = std::variant<int64_t, int32_t, half, float, nv_bfloat16>;
 
     error_t
     validate() const {
@@ -123,6 +123,13 @@ class Tensor_attributes {
         data_type    = DataType_t::INT32;
     }
 
+    Tensor_attributes(int64_t const& scalar) {
+        pass_by_value    = scalar;
+        is_pass_by_value = true;
+        dim = stride = {1};
+        data_type    = DataType_t::INT64;
+    }
+
     std::string
     get_name() const {
         return name;
@@ -156,6 +163,15 @@ class Tensor_attributes {
         return *this;
     }
 
+    int64_t
+    get_volume() const {
+        int64_t volume = 1ul;
+        for (int64_t d : dim) {
+            volume *= d;
+        }
+        return volume;
+    }
+
     std::vector<int64_t>
     get_stride() const {
         return stride;
@@ -712,12 +728,30 @@ class Matmul_fp8_attributes : public Attributes<Matmul_fp8_attributes> {
     double padding_value = 0.0;
 
    public:
-    enum class input_names { Descale_A, Descale_B, A, B, Scale_C };
+    enum class input_names { Descale_A, Descale_B, A, B, M_override, N_override, K_override, Scale_C };
     std::unordered_map<input_names, std::shared_ptr<Tensor_attributes>> inputs;
     enum class output_names { C, Amax_C };
     std::unordered_map<output_names, std::shared_ptr<Tensor_attributes>> outputs;
     NLOHMANN_DEFINE_TYPE_INTRUSIVE(Matmul_fp8_attributes, name, compute_data_type, inputs, outputs)
 
+    Matmul_fp8_attributes&
+    set_m_override(std::shared_ptr<Tensor_attributes> const& value) {
+        inputs[input_names::M_override] = value;
+        return *this;
+    }
+
+    Matmul_fp8_attributes&
+    set_n_override(std::shared_ptr<Tensor_attributes> const& value) {
+        inputs[input_names::N_override] = value;
+        return *this;
+    }
+
+    Matmul_fp8_attributes&
+    set_k_override(std::shared_ptr<Tensor_attributes> const& value) {
+        inputs[input_names::K_override] = value;
+        return *this;
+    }
+
     Matmul_fp8_attributes&
     set_padding(double const padding_val) {
         padding_value = padding_val;
@@ -1381,6 +1415,7 @@ class SDPA_attributes : public Attributes<SDPA_attributes> {
     std::optional<int> sliding_window_length;
     std::optional<float> dropout_probability;
     std::optional<float> attn_scale_value;
+    std::optional<int> max_seq_len_kv;
 
    public:
     enum class input_names {
@@ -1395,6 +1430,8 @@ class SDPA_attributes : public Attributes<SDPA_attributes> {
         Offset,
         Dropout_mask,
         Dropout_scale,
+        Page_table_K,
+        Page_table_V
     };
     std::unordered_map<input_names, std::shared_ptr<Tensor_attributes>> inputs;
     enum class output_names { O, Stats, RNG_DUMP };
@@ -1420,7 +1457,7 @@ class SDPA_attributes : public Attributes<SDPA_attributes> {
 
     SDPA_attributes&
     set_attn_scale(std::shared_ptr<Tensor_attributes> value) {
-        inputs[SDPA_attributes::input_names::Attn_scale] = value;
+        inputs[SDPA_attributes::input_names::Attn_scale] = std::move(value);
         return *this;
     }
 
@@ -1432,7 +1469,7 @@ class SDPA_attributes : public Attributes<SDPA_attributes> {
 
     SDPA_attributes&
     set_bias(std::shared_ptr<Tensor_attributes> value) {
-        inputs[SDPA_attributes::input_names::Bias] = value;
+        inputs[SDPA_attributes::input_names::Bias] = std::move(value);
         return *this;
     }
 
@@ -1450,13 +1487,13 @@ class SDPA_attributes : public Attributes<SDPA_attributes> {
 
     SDPA_attributes&
     set_seq_len_q(std::shared_ptr<Tensor_attributes> value) {
-        inputs[SDPA_attributes::input_names::SEQ_LEN_Q] = value;
+        inputs[SDPA_attributes::input_names::SEQ_LEN_Q] = std::move(value);
         return *this;
     }
 
     SDPA_attributes&
     set_seq_len_kv(std::shared_ptr<Tensor_attributes> value) {
-        inputs[SDPA_attributes::input_names::SEQ_LEN_KV] = value;
+        inputs[SDPA_attributes::input_names::SEQ_LEN_KV] = std::move(value);
         return *this;
     }
 
@@ -1483,22 +1520,40 @@ class SDPA_attributes : public Attributes<SDPA_attributes> {
                 std::shared_ptr<Tensor_attributes> seed,
                 std::shared_ptr<Tensor_attributes> offset) {
         dropout_probability                          = probability;
-        inputs[SDPA_attributes::input_names::Seed]   = seed;
-        inputs[SDPA_attributes::input_names::Offset] = offset;
+        inputs[SDPA_attributes::input_names::Seed]   = std::move(seed);
+        inputs[SDPA_attributes::input_names::Offset] = std::move(offset);
         return *this;
     }
 
     SDPA_attributes&
     set_dropout(std::shared_ptr<Tensor_attributes> mask, std::shared_ptr<Tensor_attributes> scale) {
-        inputs[SDPA_attributes::input_names::Dropout_mask]  = mask;
-        inputs[SDPA_attributes::input_names::Dropout_scale] = scale;
+        inputs[SDPA_attributes::input_names::Dropout_mask]  = std::move(mask);
+        inputs[SDPA_attributes::input_names::Dropout_scale] = std::move(scale);
         return *this;
     }
 
     // For debugging purposes only.
     SDPA_attributes&
     set_rng_dump(std::shared_ptr<Tensor_attributes> value) {
-        outputs[SDPA_attributes::output_names::RNG_DUMP] = value;
+        outputs[SDPA_attributes::output_names::RNG_DUMP] = std::move(value);
+        return *this;
+    }
+
+    SDPA_attributes&
+    set_paged_attention_k_table(std::shared_ptr<Tensor_attributes> value) {
+        inputs[SDPA_attributes::input_names::Page_table_K] = std::move(value);
+        return *this;
+    }
+
+    SDPA_attributes&
+    set_paged_attention_v_table(std::shared_ptr<Tensor_attributes> value) {
+        inputs[SDPA_attributes::input_names::Page_table_V] = std::move(value);
+        return *this;
+    }
+
+    SDPA_attributes&
+    set_paged_attention_max_seq_len_kv(int const value) {
+        max_seq_len_kv = value;
         return *this;
     }
 };
@@ -1509,7 +1564,9 @@ class SDPA_fp8_attributes : public Attributes<SDPA_fp8_attributes> {
     friend class Graph;
 
     std::optional<bool> is_inference;
-    bool causal_mask = false;
+    bool padding_mask = false;
+    bool causal_mask  = false;
+    std::optional<float> dropout_probability;
     std::optional<float> attn_scale_value;
 
    public:
@@ -1518,6 +1575,14 @@ class SDPA_fp8_attributes : public Attributes<SDPA_fp8_attributes> {
         K,
         V,
         Attn_scale,
+        Bias,
+        SEQ_LEN_Q,
+        SEQ_LEN_KV,
+        Seed,
+        Offset,
+        Dropout_mask,
+        Dropout_scale,
+
         Descale_Q,
         Descale_K,
         Descale_V,
@@ -1535,7 +1600,9 @@ class SDPA_fp8_attributes : public Attributes<SDPA_fp8_attributes> {
                                    inputs,
                                    outputs,
                                    is_inference,
+                                   padding_mask,
                                    causal_mask,
+                                   dropout_probability,
                                    attn_scale_value)
 
     SDPA_fp8_attributes&
@@ -1556,11 +1623,52 @@ class SDPA_fp8_attributes : public Attributes<SDPA_fp8_attributes> {
         return *this;
     }
 
+    SDPA_fp8_attributes&
+    set_bias(std::shared_ptr<Tensor_attributes> value) {
+        inputs[SDPA_fp8_attributes::input_names::Bias] = value;
+        return *this;
+    }
+
+    SDPA_fp8_attributes&
+    set_padding_mask(bool const value) {
+        padding_mask = value;
+        return *this;
+    }
+
+    SDPA_fp8_attributes&
+    set_seq_len_q(std::shared_ptr<Tensor_attributes> value) {
+        inputs[SDPA_fp8_attributes::input_names::SEQ_LEN_Q] = value;
+        return *this;
+    }
+
+    SDPA_fp8_attributes&
+    set_seq_len_kv(std::shared_ptr<Tensor_attributes> value) {
+        inputs[SDPA_fp8_attributes::input_names::SEQ_LEN_KV] = value;
+        return *this;
+    }
+
     SDPA_fp8_attributes&
     set_causal_mask(bool const value) {
         causal_mask = value;
         return *this;
     }
+
+    SDPA_fp8_attributes&
+    set_dropout(float const probability,
+                std::shared_ptr<Tensor_attributes> seed,
+                std::shared_ptr<Tensor_attributes> offset) {
+        dropout_probability                              = probability;
+        inputs[SDPA_fp8_attributes::input_names::Seed]   = seed;
+        inputs[SDPA_fp8_attributes::input_names::Offset] = offset;
+        return *this;
+    }
+
+    SDPA_fp8_attributes&
+    set_dropout(std::shared_ptr<Tensor_attributes> mask, std::shared_ptr<Tensor_attributes> scale) {
+        inputs[SDPA_fp8_attributes::input_names::Dropout_mask]  = mask;
+        inputs[SDPA_fp8_attributes::input_names::Dropout_scale] = scale;
+        return *this;
+    }
 };
 
 class SDPA_backward_attributes : public Attributes<SDPA_backward_attributes> {
@@ -1577,6 +1685,9 @@ class SDPA_backward_attributes : public Attributes<SDPA_backward_attributes> {
     std::optional<float> dropout_probability;
     std::optional<float> attn_scale_value;
 
+    std::optional<int64_t> max_total_seq_len_q;
+    std::optional<int64_t> max_total_seq_len_kv;
+
     bool is_deterministic_algorithm = false;
 
    public:
@@ -1661,6 +1772,18 @@ class SDPA_backward_attributes : public Attributes<SDPA_backward_attributes> {
         return *this;
     }
 
+    SDPA_backward_attributes&
+    set_max_total_seq_len_q(int64_t const value) {
+        max_total_seq_len_q = value;
+        return *this;
+    }
+
+    SDPA_backward_attributes&
+    set_max_total_seq_len_kv(int64_t const value) {
+        max_total_seq_len_kv = value;
+        return *this;
+    }
+
     SDPA_backward_attributes&
     set_causal_mask(bool const value) {
         causal_mask = value;
@@ -1718,7 +1841,10 @@ class SDPA_fp8_backward_attributes : public Attributes<SDPA_fp8_backward_attribu
     friend class SDPAFP8BackwardNode;
     friend class Graph;
 
-    bool causal_mask = false;
+    bool padding_mask = false;
+    bool causal_mask  = false;
+
+    std::optional<float> dropout_probability;
     std::optional<float> attn_scale_value;
 
    public:
@@ -1730,6 +1856,15 @@ class SDPA_fp8_backward_attributes : public Attributes<SDPA_fp8_backward_attribu
         dO,
         Stats,
         Attn_scale,
+        Bias,
+        SEQ_LEN_Q,
+        SEQ_LEN_KV,
+        Seed,
+        Offset,
+        Dropout_mask,
+        Dropout_scale,
+        Dropout_scale_inv,
+
         Descale_Q,
         Descale_K,
         Descale_V,
@@ -1753,7 +1888,9 @@ class SDPA_fp8_backward_attributes : public Attributes<SDPA_fp8_backward_attribu
                                    compute_data_type,
                                    inputs,
                                    outputs,
+                                   padding_mask,
                                    causal_mask,
+                                   dropout_probability,
                                    attn_scale_value)
 
     SDPA_fp8_backward_attributes&
@@ -1768,11 +1905,55 @@ class SDPA_fp8_backward_attributes : public Attributes<SDPA_fp8_backward_attribu
         return *this;
     }
 
+    SDPA_fp8_backward_attributes&
+    set_bias(std::shared_ptr<Tensor_attributes> value) {
+        inputs[SDPA_fp8_backward_attributes::input_names::Bias] = value;
+        return *this;
+    }
+
+    SDPA_fp8_backward_attributes&
+    set_padding_mask(bool const value) {
+        padding_mask = value;
+        return *this;
+    }
+
+    SDPA_fp8_backward_attributes&
+    set_seq_len_q(std::shared_ptr<Tensor_attributes> value) {
+        inputs[SDPA_fp8_backward_attributes::input_names::SEQ_LEN_Q] = value;
+        return *this;
+    }
+
+    SDPA_fp8_backward_attributes&
+    set_seq_len_kv(std::shared_ptr<Tensor_attributes> value) {
+        inputs[SDPA_fp8_backward_attributes::input_names::SEQ_LEN_KV] = value;
+        return *this;
+    }
+
     SDPA_fp8_backward_attributes&
     set_causal_mask(bool const value) {
         causal_mask = value;
         return *this;
     }
+
+    SDPA_fp8_backward_attributes&
+    set_dropout(float const probability,
+                std::shared_ptr<Tensor_attributes> seed,
+                std::shared_ptr<Tensor_attributes> offset) {
+        dropout_probability                                       = probability;
+        inputs[SDPA_fp8_backward_attributes::input_names::Seed]   = seed;
+        inputs[SDPA_fp8_backward_attributes::input_names::Offset] = offset;
+        return *this;
+    }
+
+    SDPA_fp8_backward_attributes&
+    set_dropout(std::shared_ptr<Tensor_attributes> mask,
+                std::shared_ptr<Tensor_attributes> scale,
+                std::shared_ptr<Tensor_attributes> scale_inv) {
+        inputs[SDPA_fp8_backward_attributes::input_names::Dropout_mask]      = mask;
+        inputs[SDPA_fp8_backward_attributes::input_names::Dropout_scale]     = scale;
+        inputs[SDPA_fp8_backward_attributes::input_names::Dropout_scale_inv] = scale_inv;
+        return *this;
+    }
 };
 
 using Scaled_dot_product_flash_attention_attributes [[deprecated]]          = SDPA_attributes;
@@ -1936,6 +2117,19 @@ class Slice_attributes : public Attributes<Slice_attributes> {
     }
 };
 
+class PagedCacheLoad_attributes : public Attributes<PagedCacheLoad_attributes> {
+    friend class Attributes<PagedCacheLoad_attributes>;
+    friend class PagedCacheLoadNode;
+    friend class INode;
+
+   public:
+    enum class input_names { container, seqLen, pageTable };
+    std::unordered_map<input_names, std::shared_ptr<Tensor_attributes>> inputs;
+    enum class output_names { yOut };
+    std::unordered_map<output_names, std::shared_ptr<Tensor_attributes>> outputs;
+    NLOHMANN_DEFINE_TYPE_INTRUSIVE(PagedCacheLoad_attributes, name, compute_data_type, inputs, outputs)
+};
+
 }  // namespace graph
 
 }  // namespace cudnn_frontend
\ No newline at end of file
