@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2022, NVIDIA CORPORATION. All rights reserved.
+ * Copyright (c) 2024, NVIDIA CORPORATION. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -24,13 +24,181 @@
 
 #include <random>
 
-#include "../../utils/helpers.h"
+#include "../utils/helpers.h"
 
 #include <cudnn_frontend.h>
 
+void
+matmul_dynamic_shapes(bool use_abs = false, bool use_bias = false) {
+    if (is_arch_supported_by_cudnn() == false) {
+        SKIP("Architecture is not supported by current cudnn version");
+    }
+    namespace fe = cudnn_frontend;
+
+    // clang-format off
+    struct {
+        int64_t b,    m,    n,    k;
+    } matmul_shapes[] = {
+        {      16,   32,   32,  128},
+        {      16,   64,   64,  128},
+        {      16,   80,   80,  128},
+        {      32,  128,  128,  256},
+        {      32,   64,   64,  256},
+    };
+    // clang-format on
+
+    constexpr int matmul_shapes_count = sizeof(matmul_shapes) / sizeof(matmul_shapes[0]);
+    int64_t max_a_volume = 0, max_b_volume = 0, max_c_volume = 0, max_bias_volume = 0;
+    for (int idx_shape = 0; idx_shape < matmul_shapes_count; ++idx_shape) {
+        const auto& matmul_shape = matmul_shapes[idx_shape];
+        max_a_volume             = std::max(max_a_volume, matmul_shape.b * matmul_shape.m * matmul_shape.k);
+        max_b_volume             = std::max(max_b_volume, matmul_shape.b * matmul_shape.k * matmul_shape.n);
+        max_c_volume             = std::max(max_c_volume, matmul_shape.b * matmul_shape.m * matmul_shape.n);
+        max_bias_volume          = std::max(max_bias_volume, matmul_shape.b * matmul_shape.m);
+    }
+
+    auto kernel_cache = std::make_shared<fe::KernelCache>();
+
+    const auto build_new_graph = [&matmul_shapes, &kernel_cache, &use_abs, &use_bias](cudnnHandle_t handle,
+                                                                                      int idx_shape) {
+        const auto& matmul_shape = matmul_shapes[idx_shape];
+
+        // Make cudnn graph
+        fe::graph::Graph graph{};
+
+        graph.set_dynamic_shape_enabled(true).set_kernel_cache(kernel_cache);
+
+        // Create the two non-virtual input tensors A and B.
+        // There are read from global memory.
+        auto A_attributes = fe::graph::Tensor_attributes()
+                                .set_name("A")
+                                .set_dim({matmul_shape.b, matmul_shape.m, matmul_shape.k})
+                                .set_stride({matmul_shape.m * matmul_shape.k, matmul_shape.k, 1})
+                                .set_data_type(fe::DataType_t::BFLOAT16);
+        auto A = graph.tensor(A_attributes);
+
+        auto B_attributes = fe::graph::Tensor_attributes()
+                                .set_name("B")
+                                .set_dim({matmul_shape.b, matmul_shape.k, matmul_shape.n})
+                                .set_stride({matmul_shape.k * matmul_shape.n, matmul_shape.n, 1})
+                                .set_data_type(fe::DataType_t::BFLOAT16);
+        auto B = graph.tensor(B_attributes);
+
+        auto matmul_attributes = fe::graph::Matmul_attributes().set_compute_data_type(fe::DataType_t::FLOAT);
+
+        std::shared_ptr<fe::graph::Tensor_attributes> C;
+        std::shared_ptr<fe::graph::Tensor_attributes> Bias;
+
+        if (use_abs) {
+            // Add abs operation
+            auto pw_0_attributes = fe::graph::Pointwise_attributes()
+                                       .set_name("pw0_Abs")
+                                       .set_mode(fe::PointwiseMode_t::ABS)
+                                       .set_compute_data_type(fe::DataType_t::FLOAT);
+
+            auto A_after_pw_0 = graph.pointwise(A, pw_0_attributes);
+            A_after_pw_0->set_data_type(fe::DataType_t::BFLOAT16);
+
+            C = graph.matmul(A_after_pw_0, B, matmul_attributes);
+        } else if (use_bias) {
+            // Create Bias vector
+            auto Bias_attributes = fe::graph::Tensor_attributes()
+                                       .set_name("Bias")
+                                       .set_dim({matmul_shape.b, matmul_shape.m, 1})
+                                       .set_stride({matmul_shape.m, 1, 1})
+                                       .set_data_type(fe::DataType_t::BFLOAT16);
+            Bias = graph.tensor(Bias_attributes);
+
+            // Add ADD operation
+            auto pw_0_attributes = fe::graph::Pointwise_attributes()
+                                       .set_name("pw0_Add")
+                                       .set_mode(fe::PointwiseMode_t::ADD)
+                                       .set_compute_data_type(fe::DataType_t::FLOAT);
+
+            auto A_after_pw_0 = graph.pointwise(A, Bias, pw_0_attributes);
+            A_after_pw_0->set_data_type(fe::DataType_t::BFLOAT16);
+
+            C = graph.matmul(A_after_pw_0, B, matmul_attributes);
+        } else {
+            C = graph.matmul(A, B, matmul_attributes);
+        }
+        C->set_output(true).set_data_type(fe::DataType_t::FLOAT);
+
+        std::cout << graph << std::endl;
+        auto status = graph.validate();
+        if (cudnnGetVersion() >= 90400) {
+            REQUIRE(status.is_good());
+        } else {
+            REQUIRE(status.is_bad());
+            SKIP("Dynamic shapes not supported pre 9.4");
+        }
+
+        status = graph.build_operation_graph(handle);
+        if (cudnnGetVersion() >= 90400) {
+            REQUIRE(status.is_good());
+        } else {
+            REQUIRE(status.is_bad());
+            SKIP("Kernel cache not supported pre 9.4");
+        }
+
+        REQUIRE(graph.create_execution_plans({fe::HeurMode_t::A}).is_good());
+
+        REQUIRE(graph.check_support(handle).is_good());
+
+        REQUIRE(graph.build_plans(handle, fe::BuildPlanPolicy_t::ALL).is_good());
+
+        return std::make_tuple(graph, A, B, C, Bias);
+    };
+
+    // Run cudnn graph
+    cudnnHandle_t handle;
+    CUDNN_CHECK(cudnnCreate(&handle));
+
+    for (int idx_shape = 0; idx_shape < matmul_shapes_count; idx_shape++) {
+        auto [graph, A, B, C, Bias] = build_new_graph(handle, idx_shape);
+        // Initialize input tensors
+        Surface<half> A_gpu(max_a_volume, false);
+        Surface<half> B_gpu(max_b_volume, false);
+        Surface<float> C_gpu(max_c_volume, false);
+        Surface<half> Bias_gpu(max_bias_volume, false);
+        Surface<int8_t> workspace(graph.get_workspace_size(), false);
+
+        std::unordered_map<std::shared_ptr<fe::graph::Tensor_attributes>, void*> variant_pack;
+        if (use_bias) {
+            variant_pack = {{A, A_gpu.devPtr}, {B, B_gpu.devPtr}, {C, C_gpu.devPtr}, {Bias, Bias_gpu.devPtr}};
+        } else {
+            variant_pack = {{A, A_gpu.devPtr}, {B, B_gpu.devPtr}, {C, C_gpu.devPtr}};
+        }
+        REQUIRE(graph.execute(handle, variant_pack, workspace.devPtr).is_good());
+    }
+
+    CUDNN_CHECK(cudnnDestroy(handle));
+}
+
+TEST_CASE("Matmul dynamic shape", "[matmul][graph][dynamic_shape]") {
+    if (cudnnGetCudartVersion() < 12000) {
+        SKIP("Test requires cuda toolkit 12.0 or above");
+    }
+    matmul_dynamic_shapes(false, false);  // Matmul dynamic shape, no abs or bias
+}
+
+TEST_CASE("Abs + Matmul dynamic shape", "[matmul][graph][dynamic_shape]") {
+    if (cudnnGetCudartVersion() < 12000) {
+        SKIP("Test requires cuda toolkit 12.0 or above");
+    }
+    matmul_dynamic_shapes(true, false);  // Matmul with abs
+}
+
+TEST_CASE("Bias + Matmul dynamic shape", "[matmul][graph][dynamic_shape]") {
+    if (cudnnGetCudartVersion() < 12000) {
+        SKIP("Test requires cuda toolkit 12.0 or above");
+    }
+    matmul_dynamic_shapes(false, true);  // Matmul with bias
+}
+
 TEST_CASE("Matmul", "[matmul][graph]") {
     if (is_arch_supported_by_cudnn() == false) {
-        SKIP("Architecture is not supported by currend cudnn version");
+        SKIP("Architecture is not supported by current cudnn version");
     }
     namespace fe = cudnn_frontend;
 
@@ -70,7 +238,7 @@ TEST_CASE("Matmul", "[matmul][graph]") {
     REQUIRE(graph.validate().is_good());
 
     cudnnHandle_t handle;
-    checkCudnnErr(cudnnCreate(&handle));
+    CUDNN_CHECK(cudnnCreate(&handle));
 
     REQUIRE(graph.build_operation_graph(handle).is_good());
     REQUIRE(graph.create_execution_plans({fe::HeurMode_t::A}).is_good());
@@ -82,11 +250,14 @@ TEST_CASE("Matmul", "[matmul][graph]") {
 
     // Run cudnn graph
     Surface<float> C_gpu(b * m * n, false);
-    Surface<int8_t> workspace(graph.get_workspace_size(), false);
+    int64_t workspace_size;
+    REQUIRE(graph.get_workspace_size(workspace_size).is_good());
+    Surface<int8_t> workspace(workspace_size, false);
+
     std::unordered_map<std::shared_ptr<fe::graph::Tensor_attributes>, void*> variant_pack = {
         {A, A_gpu.devPtr}, {B, B_gpu.devPtr}, {C, C_gpu.devPtr}};
     REQUIRE(graph.execute(handle, variant_pack, workspace.devPtr).is_good());
-    checkCudnnErr(cudnnDestroy(handle));
+    CUDNN_CHECK(cudnnDestroy(handle));
 }
 
 TEST_CASE("Abs + Matmul", "[matmul][graph]") {
@@ -136,9 +307,10 @@ TEST_CASE("Abs + Matmul", "[matmul][graph]") {
     REQUIRE(graph.validate().is_good());
 
     cudnnHandle_t handle;
-    checkCudnnErr(cudnnCreate(&handle));
+    CUDNN_CHECK(cudnnCreate(&handle));
 
     REQUIRE(graph.build_operation_graph(handle).is_good());
+
     REQUIRE(graph.create_execution_plans({fe::HeurMode_t::A}).is_good());
 
     REQUIRE(graph.check_support(handle).is_good());
@@ -147,11 +319,14 @@ TEST_CASE("Abs + Matmul", "[matmul][graph]") {
 
     // Run cudnn graph
     Surface<float> C_gpu(b * m * n, false);
-    Surface<int8_t> workspace(graph.get_workspace_size(), false);
+    int64_t workspace_size;
+    REQUIRE(graph.get_workspace_size(workspace_size).is_good());
+    Surface<int8_t> workspace(workspace_size, false);
+
     std::unordered_map<std::shared_ptr<fe::graph::Tensor_attributes>, void*> variant_pack = {
         {A, A_gpu.devPtr}, {B, B_gpu.devPtr}, {C, C_gpu.devPtr}};
     REQUIRE(graph.execute(handle, variant_pack, workspace.devPtr).is_good());
-    checkCudnnErr(cudnnDestroy(handle));
+    CUDNN_CHECK(cudnnDestroy(handle));
 }
 
 TEST_CASE("Bias + Matmul", "[matmul][graph]") {
@@ -217,7 +392,7 @@ TEST_CASE("Bias + Matmul", "[matmul][graph]") {
     REQUIRE(graph.validate().is_good());
 
     cudnnHandle_t handle;
-    checkCudnnErr(cudnnCreate(&handle));
+    CUDNN_CHECK(cudnnCreate(&handle));
 
     REQUIRE(graph.build_operation_graph(handle).is_good());
     REQUIRE(graph.create_execution_plans({fe::HeurMode_t::A}).is_good());
@@ -260,7 +435,7 @@ TEST_CASE("Bias + Matmul", "[matmul][graph]") {
                 std::mt19937{std::random_device{}()});
     Surface<int8_t> workspace(graph.get_workspace_size_plan_at_index(random_successful.front()), false);
     REQUIRE(graph.execute_plan_at_index(handle, variant_pack, workspace.devPtr, random_successful.front()).is_good());
-    checkCudnnErr(cudnnDestroy(handle));
+    CUDNN_CHECK(cudnnDestroy(handle));
 }
 
 TEST_CASE("Matmul SBR Graph", "[matmul][graph]") {
@@ -353,7 +528,7 @@ TEST_CASE("Matmul SBR Graph", "[matmul][graph]") {
         };
 
     cudnnHandle_t handle;
-    checkCudnnErr(cudnnCreate(&handle));
+    CUDNN_CHECK(cudnnCreate(&handle));
 
     Surface<half> x_tensor(4 * 16 * 64, false);
     Surface<half> w_tensor(4 * 64 * 32, false);
@@ -364,7 +539,9 @@ TEST_CASE("Matmul SBR Graph", "[matmul][graph]") {
     auto [graph, A, B, bias, scale, O] = lookup_cache_or_build_graph(
         handle, x_tensor.devPtr, w_tensor.devPtr, s_tensor.devPtr, b_tensor.devPtr, y_tensor.devPtr);
 
-    Surface<int8_t> workspace(graph->get_workspace_size(), false);
+    int64_t workspace_size;
+    REQUIRE(graph->get_workspace_size(workspace_size).is_good());
+    Surface<int8_t> workspace(workspace_size, false);
 
     std::unordered_map<std::shared_ptr<fe::graph::Tensor_attributes>, void*> variant_pack = {{A, x_tensor.devPtr},
                                                                                              {B, w_tensor.devPtr},
@@ -417,7 +594,7 @@ TEST_CASE("Matmul with restricted shared memory", "[matmul][graph]") {
     REQUIRE(graph.validate().is_good());
 
     cudnnHandle_t handle;
-    checkCudnnErr(cudnnCreate(&handle));
+    CUDNN_CHECK(cudnnCreate(&handle));
 
     REQUIRE(graph.build_operation_graph(handle).is_good());
     REQUIRE(graph.create_execution_plans({fe::HeurMode_t::A}).is_good());
@@ -429,9 +606,12 @@ TEST_CASE("Matmul with restricted shared memory", "[matmul][graph]") {
 
     // Run cudnn graph
     Surface<float> C_gpu(b * m * n, false);
-    Surface<int8_t> workspace(graph.get_workspace_size(), false);
+    int64_t workspace_size;
+    REQUIRE(graph.get_workspace_size(workspace_size).is_good());
+    Surface<int8_t> workspace(workspace_size, false);
+
     std::unordered_map<std::shared_ptr<fe::graph::Tensor_attributes>, void*> variant_pack = {
         {A, A_gpu.devPtr}, {B, B_gpu.devPtr}, {C, C_gpu.devPtr}};
     REQUIRE(graph.execute(handle, variant_pack, workspace.devPtr).is_good());
-    checkCudnnErr(cudnnDestroy(handle));
+    CUDNN_CHECK(cudnnDestroy(handle));
 }
\ No newline at end of file
